@@ -69,8 +69,6 @@ G_BEGIN_DECLS
 #define TERMINAL_APP_ENABLE_MNEMONICS       "enable-mnemonics"
 #define TERMINAL_APP_SYSTEM_FONT            "system-font"
 
-#define EPSILON (1.0e-6)
-
 /* TerminalApp */
 
 #define TERMINAL_TYPE_APP              (terminal_app_get_type ())
@@ -93,11 +91,13 @@ TerminalApp* terminal_app_get (void);
 
 gboolean terminal_app_handle_options (TerminalApp *app,
                                       TerminalOptions *options,
+                                      gboolean allow_resume,
                                       GError **error);
 
 void terminal_app_edit_profile (TerminalApp     *app,
                                 TerminalProfile *profile,
-                                GtkWindow       *transient_parent);
+                                GtkWindow       *transient_parent,
+                                const char      *widget_name);
 
 void terminal_app_new_profile (TerminalApp     *app,
                                TerminalProfile *default_base_profile,
