@@ -18,8 +18,6 @@
 
 #include <config.h>
 
-#undef VTE_DISABLE_DEPRECATED
-
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -42,6 +40,11 @@
 #include "terminal-util.h"
 #include "terminal-window.h"
 
+#if GTK_CHECK_VERSION (2, 18, 0)
+#include "terminal-info-bar.h"
+#define USE_INFOBARS
+#endif
+
 #include "eggshell.h"
 
 #define URL_MATCH_CURSOR  (GDK_HAND2)
@@ -55,7 +58,6 @@ typedef struct
 
 struct _TerminalScreenPrivate
 {
-  TerminalWindow *window;
   TerminalProfile *profile; /* may be NULL at times */
   guint profile_changed_id;
   guint profile_forgotten_id;
@@ -240,62 +242,6 @@ free_tag_data (TagData *tagdata)
   g_slice_free (TagData, tagdata);
 }
 
-#ifdef GNOME_ENABLE_DEBUG
-static void
-parent_size_request (GtkWidget *scrolled_window, GtkRequisition *req, GtkWidget *screen)
-{
-  _terminal_debug_print (TERMINAL_DEBUG_GEOMETRY,
-                         "[screen %p] scrolled-window size req %d : %d\n",
-                         screen, req->width, req->height);
-}
-#endif
-
-static void
-parent_parent_set_cb (GtkWidget *widget,
-                      GtkWidget *old_parent,
-                      TerminalScreen *screen)
-{
-  TerminalScreenPrivate *priv = screen->priv;
-  GtkWidget *toplevel;
-
-  if (widget->parent)
-    {
-      g_return_if_fail (GTK_IS_NOTEBOOK (widget->parent));
-
-      toplevel = gtk_widget_get_toplevel (widget);
-#if GTK_CHECK_VERSION (2, 19, 3)
-      g_return_if_fail (gtk_widget_is_toplevel (toplevel));
-#else
-      g_return_if_fail (GTK_WIDGET_TOPLEVEL (toplevel));
-#endif
-
-      priv->window = TERMINAL_WINDOW (toplevel);
-    }
-  else
-    priv->window = NULL;
-}
-
-static void
-parent_set_callback (GtkWidget *widget,
-                     GtkWidget *old_parent)
-{
-  if (old_parent)
-    g_signal_handlers_disconnect_by_func (old_parent, G_CALLBACK (parent_parent_set_cb), widget);
-
-  if (widget->parent)
-    g_signal_connect (widget->parent, "parent-set", G_CALLBACK (parent_parent_set_cb), widget);
-
-#ifdef GNOME_ENABLE_DEBUG
-  _TERMINAL_DEBUG_IF (TERMINAL_DEBUG_GEOMETRY)
-    {
-      if (old_parent)
-        g_signal_handlers_disconnect_by_func (old_parent, G_CALLBACK (parent_size_request), widget);
-      if (widget->parent)
-        g_signal_connect (widget->parent, "size-request", G_CALLBACK (parent_size_request), widget);
-    }
-#endif
-}
-
 static void
 terminal_screen_class_enable_menu_bar_accel_notify_cb (TerminalApp *app,
                                                        GParamSpec *pspec,
@@ -320,6 +266,35 @@ terminal_screen_class_enable_menu_bar_accel_notify_cb (TerminalApp *app,
     gtk_binding_entry_skip (binding_set, GDK_F10, GDK_SHIFT_MASK);
 }
 
+static TerminalWindow *
+terminal_screen_get_window (TerminalScreen *screen)
+{
+  GtkWidget *widget = GTK_WIDGET (screen);
+  GtkWidget *toplevel;
+
+  toplevel = gtk_widget_get_toplevel (widget);
+#if GTK_CHECK_VERSION (2, 18, 0)
+  if (!gtk_widget_is_toplevel (toplevel))
+#else
+  if (!GTK_WIDGET_TOPLEVEL (toplevel))
+#endif
+    return NULL;
+
+  return TERMINAL_WINDOW (toplevel);
+}
+
+static gboolean
+window_uses_argb_visual (TerminalScreen *screen)
+{
+  TerminalWindow *window;
+
+  window = terminal_screen_get_window (screen);
+  if (window == NULL || !gtk_widget_get_realized (GTK_WIDGET (window)))
+    return FALSE;
+
+  return terminal_window_uses_argb_visual (window);
+}
+
 static void
 terminal_screen_realize (GtkWidget *widget)
 {
@@ -329,13 +304,11 @@ terminal_screen_realize (GtkWidget *widget)
 
   GTK_WIDGET_CLASS (terminal_screen_parent_class)->realize (widget);
 
-  g_assert (priv->window != NULL);
-
   /* FIXME: Don't enable this if we have a compmgr. */
   bg_type = terminal_profile_get_property_enum (priv->profile, TERMINAL_PROFILE_BACKGROUND_TYPE);
   vte_terminal_set_background_transparent (VTE_TERMINAL (screen),
                                            bg_type == TERMINAL_BACKGROUND_TRANSPARENT &&
-                                           !terminal_window_uses_argb_visual (priv->window));
+                                           !window_uses_argb_visual (screen));
 }
 
 static void
@@ -350,7 +323,7 @@ terminal_screen_style_set (GtkWidget *widget,
 
   update_color_scheme (screen);
 
-  if (GTK_WIDGET_REALIZED (widget))
+  if (gtk_widget_get_realized (widget))
     terminal_screen_change_font (screen);
 }
 
@@ -444,8 +417,6 @@ terminal_screen_init (TerminalScreen *screen)
   g_signal_connect (terminal_app_get (), "notify::system-font",
                     G_CALLBACK (terminal_screen_system_font_notify_cb), screen);
 
-  g_signal_connect (screen, "parent-set", G_CALLBACK (parent_set_callback), NULL);
-
 #ifdef GNOME_ENABLE_DEBUG
   _TERMINAL_DEBUG_IF (TERMINAL_DEBUG_GEOMETRY)
     {
@@ -738,11 +709,11 @@ terminal_screen_new (TerminalProfile *profile,
 
   terminal_screen_set_profile (screen, profile);
 
-#if 0
-  vte_terminal_set_size (VTE_TERMINAL (screen),
-			 terminal_profile_get_property_int (profile, TERMINAL_PROFILE_DEFAULT_SIZE_COLUMNS),
-			 terminal_profile_get_property_int (profile, TERMINAL_PROFILE_DEFAULT_SIZE_ROWS));
-#endif
+  if (terminal_profile_get_property_boolean (profile, TERMINAL_PROFILE_USE_CUSTOM_DEFAULT_SIZE)) {
+    vte_terminal_set_size (VTE_TERMINAL (screen),
+			   terminal_profile_get_property_int (profile, TERMINAL_PROFILE_DEFAULT_SIZE_COLUMNS),
+			   terminal_profile_get_property_int (profile, TERMINAL_PROFILE_DEFAULT_SIZE_ROWS));
+  }
 
   if (title)
     terminal_screen_set_override_title (screen, title);
@@ -939,6 +910,7 @@ terminal_screen_profile_notify_cb (TerminalProfile *profile,
   VteTerminal *vte_terminal = VTE_TERMINAL (screen);
   const char *prop_name;
   TerminalBackgroundType bg_type;
+  TerminalWindow *window;
 
   if (pspec)
     prop_name = pspec->name;
@@ -947,12 +919,12 @@ terminal_screen_profile_notify_cb (TerminalProfile *profile,
 
   g_object_freeze_notify (object);
 
-  if (priv->window)
+  if ((window = terminal_screen_get_window (screen)))
     {
       /* We need these in line for the set_size in
        * update_on_realize
        */
-      terminal_window_update_geometry (priv->window);
+      terminal_window_update_geometry (window);
     }
   
   if (!prop_name || prop_name == I_(TERMINAL_PROFILE_SCROLLBAR_POSITION))
@@ -966,7 +938,7 @@ terminal_screen_profile_notify_cb (TerminalProfile *profile,
       terminal_screen_cook_icon_title (screen);
     }
 
-  if (GTK_WIDGET_REALIZED (screen) &&
+  if (gtk_widget_get_realized (GTK_WIDGET (screen)) &&
       (!prop_name ||
        prop_name == I_(TERMINAL_PROFILE_USE_SYSTEM_FONT) ||
        prop_name == I_(TERMINAL_PROFILE_FONT)))
@@ -1064,7 +1036,7 @@ terminal_screen_profile_notify_cb (TerminalProfile *profile,
       /* FIXME: Don't enable this if we have a compmgr. */
       vte_terminal_set_background_transparent (vte_terminal,
                                                bg_type == TERMINAL_BACKGROUND_TRANSPARENT &&
-                                               (!priv->window || !terminal_window_uses_argb_visual (priv->window)));
+                                               !window_uses_argb_visual (screen));
     }
 
   if (!prop_name || prop_name == I_(TERMINAL_PROFILE_BACKSPACE_BINDING))
@@ -1168,7 +1140,7 @@ terminal_screen_system_font_notify_cb (TerminalApp *app,
 {
   TerminalScreenPrivate *priv = screen->priv;
 
-  if (!GTK_WIDGET_REALIZED (screen))
+  if (!gtk_widget_get_realized (GTK_WIDGET (screen)))
     return;
 
   if (!terminal_profile_get_property_boolean (priv->profile, TERMINAL_PROFILE_USE_SYSTEM_FONT))
@@ -1180,11 +1152,12 @@ terminal_screen_system_font_notify_cb (TerminalApp *app,
 static void
 terminal_screen_change_font (TerminalScreen *screen)
 {
-  TerminalScreenPrivate *priv = screen->priv;
+  TerminalWindow *window;
 
   terminal_screen_set_font (screen);
 
-  terminal_window_set_size (priv->window, screen, TRUE);
+  window = terminal_screen_get_window (screen);
+  terminal_window_set_size (window, screen, TRUE);
 }
 
 static void
@@ -1299,42 +1272,28 @@ terminal_screen_get_initial_environment (TerminalScreen *screen)
   return screen->priv->initial_env;
 }
 
-static void
-show_command_error_dialog (TerminalScreen *screen,
-                           GError         *error)
-{
-  g_assert (error != NULL);
-  
-  terminal_util_show_error_dialog ((GtkWindow*) gtk_widget_get_ancestor (GTK_WIDGET (screen), GTK_TYPE_WINDOW), NULL, error,
-                                   "%s", _("There was a problem with the command for this terminal"));
-}
-
 static gboolean
 get_child_command (TerminalScreen *screen,
                    const char     *shell_env,
-                   char          **file_p,
+                   GSpawnFlags    *spawn_flags_p,
                    char         ***argv_p,
                    GError        **err)
 {
   TerminalScreenPrivate *priv = screen->priv;
   TerminalProfile *profile;
-  char  *file;
   char **argv;
 
+  g_assert (spawn_flags_p != NULL && argv_p != NULL);
+
   profile = priv->profile;
 
-  file = NULL;
-  argv = NULL;
-  
-  if (file_p)
-    *file_p = NULL;
-  if (argv_p)
-    *argv_p = NULL;
+  *argv_p = argv = NULL;
 
   if (priv->override_command)
     {
-      file = g_strdup (priv->override_command[0]);
       argv = g_strdupv (priv->override_command);
+
+      *spawn_flags_p |= G_SPAWN_SEARCH_PATH;
     }
   else if (terminal_profile_get_property_boolean (profile, TERMINAL_PROFILE_USE_CUSTOM_COMMAND))
     {
@@ -1343,44 +1302,37 @@ get_child_command (TerminalScreen *screen,
                                err))
         return FALSE;
 
-      file = g_strdup (argv[0]);
+      *spawn_flags_p |= G_SPAWN_SEARCH_PATH;
     }
   else
     {
       const char *only_name;
       char *shell;
+      int argc = 0;
 
       shell = egg_shell (shell_env);
 
-      file = g_strdup (shell);
-      
       only_name = strrchr (shell, '/');
       if (only_name != NULL)
         only_name++;
       else
         only_name = shell;
 
-      argv = g_new (char*, 2);
+      argv = g_new (char*, 3);
+
+      argv[argc++] = shell;
 
       if (terminal_profile_get_property_boolean (profile, TERMINAL_PROFILE_LOGIN_SHELL))
-        argv[0] = g_strconcat ("-", only_name, NULL);
+        argv[argc++] = g_strconcat ("-", only_name, NULL);
       else
-        argv[0] = g_strdup (only_name);
+        argv[argc++] = g_strdup (only_name);
 
-      argv[1] = NULL;
+      argv[argc++] = NULL;
 
-      g_free (shell);
+      *spawn_flags_p |= G_SPAWN_FILE_AND_ARGV_ZERO;
     }
 
-  if (file_p)
-    *file_p = file;
-  else
-    g_free (file);
-
-  if (argv_p)
-    *argv_p = argv;
-  else
-    g_strfreev (argv);
+  *argv_p = argv;
 
   return TRUE;
 }
@@ -1438,7 +1390,7 @@ get_child_environment (TerminalScreen *screen,
   
 #ifdef GDK_WINDOWING_X11
   /* FIXME: moving the tab between windows, or the window between displays will make the next two invalid... */
-  g_hash_table_replace (env_table, g_strdup ("WINDOWID"), g_strdup_printf ("%ld", GDK_WINDOW_XWINDOW (window->window)));
+  g_hash_table_replace (env_table, g_strdup ("WINDOWID"), g_strdup_printf ("%ld", GDK_WINDOW_XWINDOW (gtk_widget_get_window (window))));
   g_hash_table_replace (env_table, g_strdup ("DISPLAY"), g_strdup (gdk_display_get_name (gtk_widget_get_display (window))));
 #endif
 
@@ -1456,6 +1408,43 @@ get_child_environment (TerminalScreen *screen,
   return (char **) g_ptr_array_free (retval, FALSE);
 }
 
+#ifdef USE_INFOBARS
+
+enum {
+  RESPONSE_RELAUNCH,
+  RESPONSE_EDIT_PROFILE
+};
+
+static void
+info_bar_response_cb (GtkWidget *info_bar,
+                      int response,
+                      TerminalScreen *screen)
+{
+  gtk_widget_grab_focus (GTK_WIDGET (screen));
+
+  switch (response) {
+    case GTK_RESPONSE_CANCEL:
+      gtk_widget_destroy (info_bar);
+      g_signal_emit (screen, signals[CLOSE_SCREEN], 0);
+      break;
+    case RESPONSE_RELAUNCH:
+      gtk_widget_destroy (info_bar);
+      terminal_screen_launch_child_on_idle (screen);
+      break;
+    case RESPONSE_EDIT_PROFILE:
+      terminal_app_edit_profile (terminal_app_get (),
+                                 terminal_screen_get_profile (screen),
+                                 GTK_WINDOW (terminal_screen_get_window (screen)),
+                                 "custom-command-entry");
+      break;
+    default:
+      gtk_widget_destroy (info_bar);
+      break;
+  }
+}
+
+#endif /* USE_INFOBARS */
+
 static gboolean
 terminal_screen_launch_child_cb (TerminalScreen *screen)
 {
@@ -1463,10 +1452,12 @@ terminal_screen_launch_child_cb (TerminalScreen *screen)
   VteTerminal *terminal = VTE_TERMINAL (screen);
   TerminalProfile *profile;
   char **env, **argv;
-  char *path, *shell = NULL;
+  char *shell = NULL;
   GError *err = NULL;
-  gboolean update_records;
   const char *working_dir;
+  VtePtyFlags pty_flags = VTE_PTY_DEFAULT;
+  GSpawnFlags spawn_flags = 0;
+  GPid pid;
 
   priv->launch_child_source_id = 0;
 
@@ -1478,45 +1469,61 @@ terminal_screen_launch_child_cb (TerminalScreen *screen)
 
   env = get_child_environment (screen, &shell);
 
-  if (!get_child_command (screen, shell, &path, &argv, &err))
-    {
-      show_command_error_dialog (screen, err);
-      g_error_free (err);
-
-      g_strfreev (env);
-      g_free (shell);
-
-      return FALSE;
-    }
-
-  update_records = terminal_profile_get_property_boolean (profile, TERMINAL_PROFILE_UPDATE_RECORDS);
-
   if (priv->initial_working_directory)
     working_dir = priv->initial_working_directory;
   else
     working_dir = g_get_home_dir ();
 
-  priv->child_pid = vte_terminal_fork_command (terminal,
-                                               path,
-                                               argv,
-                                               env,
-                                               working_dir,
-                                               terminal_profile_get_property_boolean (profile, TERMINAL_PROFILE_LOGIN_SHELL),
-                                               update_records,
-                                               update_records);
+  if (!terminal_profile_get_property_boolean (profile, TERMINAL_PROFILE_LOGIN_SHELL))
+    pty_flags |= VTE_PTY_NO_LASTLOG;
+  if (!terminal_profile_get_property_boolean (profile, TERMINAL_PROFILE_UPDATE_RECORDS))
+    pty_flags |= VTE_PTY_NO_UTMP | VTE_PTY_NO_WTMP;
 
-  if (priv->child_pid == -1)
-    {
+  if (!get_child_command (screen, shell, &spawn_flags, &argv, &err) ||
+      !vte_terminal_fork_command_full (terminal,
+                                       pty_flags,
+                                       working_dir,
+                                       argv,
+                                       env,
+                                       spawn_flags,
+                                       NULL, NULL,
+                                       &pid,
+                                       &err)) {
+#ifdef USE_INFOBARS
+    GtkWidget *info_bar;
 
-      terminal_util_show_error_dialog ((GtkWindow*) gtk_widget_get_ancestor (GTK_WIDGET (screen), GTK_TYPE_WINDOW), NULL,
-                                       err,
-                                       "%s", _("There was an error creating the child process for this terminal"));
-    }
-  
+    info_bar = terminal_info_bar_new (GTK_MESSAGE_ERROR,
+                                      _("_Profile Preferences"), RESPONSE_EDIT_PROFILE,
+                                      _("_Relaunch"), RESPONSE_RELAUNCH,
+                                      NULL);
+    terminal_info_bar_format_text (TERMINAL_INFO_BAR (info_bar),
+                                   _("There was an error creating the child process for this terminal"));
+    terminal_info_bar_format_text (TERMINAL_INFO_BAR (info_bar),
+                                   "%s", err->message);
+    g_signal_connect (info_bar, "response",
+                      G_CALLBACK (info_bar_response_cb), screen);
+
+    gtk_box_pack_start (GTK_BOX (terminal_screen_container_get_from_screen (screen)),
+                        info_bar, FALSE, FALSE, 0);
+    gtk_info_bar_set_default_response (GTK_INFO_BAR (info_bar), GTK_RESPONSE_CANCEL);
+    gtk_widget_show (info_bar);
+#else
+    terminal_util_show_error_dialog (GTK_WINDOW (terminal_screen_get_window (screen)), NULL,
+                                     err,
+                                     "%s", _("There was an error creating the child process for this terminal"));
+#endif /* USE_INFOBARS */
+
+    g_error_free (err);
+    g_strfreev (env);
+    g_free (shell);
+
+    return FALSE;
+  }
+
+  priv->child_pid = pid;
   priv->pty_fd = vte_terminal_get_pty (terminal);
 
   g_free (shell);
-  g_free (path);
   g_strfreev (argv);
   g_strfreev (env);
 
@@ -1541,13 +1548,12 @@ terminal_screen_launch_child_on_idle (TerminalScreen *screen)
 static TerminalScreenPopupInfo *
 terminal_screen_popup_info_new (TerminalScreen *screen)
 {
-  TerminalScreenPrivate *priv = screen->priv;
   TerminalScreenPopupInfo *info;
 
   info = g_slice_new0 (TerminalScreenPopupInfo);
   info->ref_count = 1;
   info->screen = g_object_ref (screen);
-  info->window = priv->window;
+  info->window = terminal_screen_get_window (screen);
 
   return info;
 }
@@ -1813,7 +1819,7 @@ terminal_screen_set_font_scale (TerminalScreen *screen,
   
   priv->font_scale = factor;
   
-  if (GTK_WIDGET_REALIZED (screen))
+  if (gtk_widget_get_realized (GTK_WIDGET (screen)))
     {
       /* Update the font */
       terminal_screen_change_font (screen);
@@ -1872,7 +1878,37 @@ terminal_screen_child_exited (VteTerminal *terminal)
     case TERMINAL_EXIT_RESTART:
       terminal_screen_launch_child_on_idle (screen);
       break;
-    case TERMINAL_EXIT_HOLD:
+#ifdef USE_INFOBARS
+    case TERMINAL_EXIT_HOLD: {
+      GtkWidget *info_bar;
+      int status;
+
+      status = vte_terminal_get_child_exit_status (terminal);
+
+      info_bar = terminal_info_bar_new (GTK_MESSAGE_INFO,
+                                        _("_Relaunch"), RESPONSE_RELAUNCH,
+                                        NULL);
+      if (WIFEXITED (status)) {
+        terminal_info_bar_format_text (TERMINAL_INFO_BAR (info_bar),
+                                      _("The child process exited normally with status %d."), WEXITSTATUS (status));
+      } else if (WIFSIGNALED (status)) {
+        terminal_info_bar_format_text (TERMINAL_INFO_BAR (info_bar),
+                                      _("The child process was terminated by signal %d."), WTERMSIG (status));
+      } else {
+        terminal_info_bar_format_text (TERMINAL_INFO_BAR (info_bar),
+                                      _("The child process was terminated."));
+      }
+      g_signal_connect (info_bar, "response",
+                        G_CALLBACK (info_bar_response_cb), screen);
+
+      gtk_box_pack_start (GTK_BOX (terminal_screen_container_get_from_screen (screen)),
+                          info_bar, FALSE, FALSE, 0);
+      gtk_info_bar_set_default_response (GTK_INFO_BAR (info_bar), RESPONSE_RELAUNCH);
+      gtk_widget_show (info_bar);
+      break;
+    }
+#endif /* USE_INFOBARS */
+
     default:
       break;
     }
@@ -1907,6 +1943,14 @@ terminal_screen_drag_data_received (GtkWidget        *widget,
 {
   TerminalScreen *screen = TERMINAL_SCREEN (widget);
   TerminalScreenPrivate *priv = screen->priv;
+  const guchar *selection_data_data;
+  GdkAtom selection_data_target;
+  gint selection_data_length, selection_data_format;
+
+  selection_data_data = gtk_selection_data_get_data (selection_data);
+  selection_data_target = gtk_selection_data_get_target (selection_data);
+  selection_data_length = gtk_selection_data_get_length (selection_data);
+  selection_data_format = gtk_selection_data_get_format (selection_data);
 
 #if 0
   {
@@ -1927,7 +1971,7 @@ terminal_screen_drag_data_received (GtkWidget        *widget,
   }
 #endif
 
-  if (gtk_targets_include_uri (&selection_data->target, 1))
+  if (gtk_targets_include_uri (&selection_data_target, 1))
     {
       char **uris;
       char *text;
@@ -1945,7 +1989,7 @@ terminal_screen_drag_data_received (GtkWidget        *widget,
 
       g_strfreev (uris);
     }
-  else if (gtk_targets_include_text (&selection_data->target, 1))
+  else if (gtk_targets_include_text (&selection_data_target, 1))
     {
       char *text;
 
@@ -1958,13 +2002,14 @@ terminal_screen_drag_data_received (GtkWidget        *widget,
     {
     case TARGET_COLOR:
       {
-        guint16 *data = (guint16 *)selection_data->data;
+        guint16 *data = (guint16 *)selection_data_data;
         GdkColor color;
 
         /* We accept drops with the wrong format, since the KDE color
          * chooser incorrectly drops application/x-color with format 8.
+         * So just check for the data length.
          */
-        if (selection_data->length != 8)
+        if (selection_data_length != 8)
           return;
 
         color.red = data[0];
@@ -1991,13 +2036,13 @@ terminal_screen_drag_data_received (GtkWidget        *widget,
          * The data contains the URL, a \n, then the
          * title of the web page.
          */
-        if (selection_data->format != 8 ||
-            selection_data->length == 0 ||
-            (selection_data->length % 2) != 0)
+        if (selection_data_format != 8 ||
+            selection_data_length == 0 ||
+            (selection_data_length % 2) != 0)
           return;
 
-        utf8_data = g_utf16_to_utf8 ((const gunichar2*) selection_data->data,
-                                     selection_data->length / 2,
+        utf8_data = g_utf16_to_utf8 ((const gunichar2*) selection_data_data,
+                                     selection_data_length / 2,
                                      NULL, NULL, NULL);
         if (!utf8_data)
           return;
@@ -2026,10 +2071,10 @@ terminal_screen_drag_data_received (GtkWidget        *widget,
         /* The data contains the URL, a \n, then the
          * title of the web page.
          */
-        if (selection_data->length < 0 || selection_data->format != 8)
+        if (selection_data_length < 0 || selection_data_format != 8)
           return;
 
-        utf8_data = g_strndup ((char *) selection_data->data, selection_data->length);
+        utf8_data = g_strndup ((char *) selection_data_data, selection_data_length);
         newline = strchr (utf8_data, '\n');
         if (newline)
           *newline = '\0';
@@ -2050,10 +2095,10 @@ terminal_screen_drag_data_received (GtkWidget        *widget,
         char *utf8_data;
         char **uris;
         
-        if (selection_data->length < 0 || selection_data->format != 8)
+        if (selection_data_length < 0 || selection_data_format != 8)
           return;
         
-        utf8_data = g_strndup ((char *) selection_data->data, selection_data->length);
+        utf8_data = g_strndup ((char *) selection_data_data, selection_data_length);
         uris = g_uri_list_extract_uris (utf8_data);
         g_free (utf8_data);
 
@@ -2094,17 +2139,17 @@ terminal_screen_drag_data_received (GtkWidget        *widget,
         GtkWidget *dest_notebook;
         int page_num;
 
-        container = *(GtkWidget**) selection_data->data;
+        container = *(GtkWidget**) selection_data_data;
         if (!GTK_IS_WIDGET (container))
           return;
 
-        moving_screen = terminal_screen_container_get_screen (container);
+        moving_screen = terminal_screen_container_get_screen (TERMINAL_SCREEN_CONTAINER (container));
         g_return_if_fail (TERMINAL_IS_SCREEN (moving_screen));
         if (!TERMINAL_IS_SCREEN (moving_screen))
           return;
 
-        source_window = moving_screen->priv->window;
-        dest_window = screen->priv->window;
+        source_window = terminal_screen_get_window (moving_screen);
+        dest_window = terminal_screen_get_window (screen);
         dest_notebook = terminal_window_get_notebook (dest_window);
         page_num = gtk_notebook_page_num (GTK_NOTEBOOK (dest_notebook), 
                                           GTK_WIDGET (screen));
@@ -2123,12 +2168,12 @@ void
 _terminal_screen_update_scrollbar (TerminalScreen *screen)
 {
   TerminalScreenPrivate *priv = screen->priv;
-  GtkWidget *parent;
+  TerminalScreenContainer *container;
   GtkPolicyType policy = GTK_POLICY_ALWAYS;
   GtkCornerType corner = GTK_CORNER_TOP_LEFT;
 
-  parent = GTK_WIDGET (screen)->parent;
-  if (!parent)
+  container = terminal_screen_container_get_from_screen (screen);
+  if (container == NULL)
     return;
 
   switch (terminal_profile_get_property_enum (priv->profile, TERMINAL_PROFILE_SCROLLBAR_POSITION))
@@ -2149,8 +2194,8 @@ _terminal_screen_update_scrollbar (TerminalScreen *screen)
       break;
     }
 
-  terminal_screen_container_set_placement (parent, corner);
-  terminal_screen_container_set_policy (parent, GTK_POLICY_NEVER, policy);
+  terminal_screen_container_set_placement (container, corner);
+  terminal_screen_container_set_policy (container, GTK_POLICY_NEVER, policy);
 }
 
 void
@@ -2160,8 +2205,8 @@ terminal_screen_get_size (TerminalScreen *screen,
 {
   VteTerminal *terminal = VTE_TERMINAL (screen);
 
-  *width_chars = terminal->column_count;
-  *height_chars = terminal->row_count;
+  *width_chars = vte_terminal_get_column_count (terminal);
+  *height_chars = vte_terminal_get_row_count (terminal);
 }
 
 void
@@ -2171,8 +2216,8 @@ terminal_screen_get_cell_size (TerminalScreen *screen,
 {
   VteTerminal *terminal = VTE_TERMINAL (screen);
 
-  *cell_width_pixels = terminal->char_width;
-  *cell_height_pixels = terminal->char_height;
+  *cell_width_pixels = vte_terminal_get_char_width (terminal);
+  *cell_height_pixels = vte_terminal_get_char_height (terminal);
 }
 
 static void
